@@ -28,6 +28,8 @@
 
 #include "ipsecmast.h"
 
+extern void child_handler (int signal);
+
 struct buffer *new_payload (struct sockaddr_in peer)
 {
     struct buffer *tmp = new_buf (MAX_RECV_SIZE);
@@ -427,33 +429,40 @@ void destroy_call (struct call *c)
     pid = c->pppd;
     if (pid)
     {
-        /* Set c->pppd to zero to prevent recursion with child_handler */
-        c->pppd = 0;
-	/* There is a bug in some pppd versions where sending a SIGTERM
-	   does not actually seem to kill pppd, and xl2tpd waits indefinately
-	   using waitpid, not accepting any new connections either. Therefor
-	   we now use some more force and send it a SIGKILL instead of SIGTERM.
-	   One confirmed buggy version of pppd is ppp-2.4.2-6.4.RHEL4
-	   See http://bugs.xelerance.com/view.php?id=739
-	*/
+      /* we'll do waitpid here so disable child_handler while at it */
+      signal (SIGCHLD, SIG_DFL);
+      /* Set c->pppd to zero to prevent recursion with child_handler */
+      c->pppd = 0;
+      /*
+       * There is a bug in some pppd versions where sending a SIGTERM
+       * does not actually seem to kill pppd, and xl2tpd waits indefinately
+       * using waitpid, not accepting any new connections either. Therefor
+       * we now use some more force and send it a SIGKILL instead of SIGTERM.
+       * One confirmed buggy version of pppd is ppp-2.4.2-6.4.RHEL4
+       * See http://bugs.xelerance.com/view.php?id=739
+       *
+       * Sometimes pppd takes 7 sec to go down! We don't have that much time,
+       * since all other calls are suspended while doing this.
+       */
 
 #ifdef TRUST_PPPD_TO_DIE
  #ifdef DEBUG_PPPD
-            l2tp_log (LOG_DEBUG, "Trustingly terminating pppd: sending TERM signal to pid %d\n", pid);
+      l2tp_log (LOG_DEBUG, "Terminating pppd: sending TERM signal to pid %d\n", pid);
  #endif
-        kill (pid, SIGTERM);
+      kill (pid, SIGTERM);
 #else
  #ifdef DEBUG_PPPD
-            l2tp_log (LOG_DEBUG, "Untrustingly terminating pppd: sending KILL signal to pid %d\n", pid);
+      l2tp_log (LOG_DEBUG, "Terminating pppd: sending KILL signal to pid %d\n", pid);
  #endif
-        kill (pid, SIGKILL);
+      kill (pid, SIGKILL);
 #endif
 
-        waitpid (pid, NULL, 0);
+      waitpid (pid, NULL, 0);
 #ifdef DEBUG_PPPD
-        l2tp_log (LOG_DEBUG, "pppd %d successfully terminated\n", pid);
+      l2tp_log (LOG_DEBUG, "pppd %d successfully terminated\n", pid);
 #endif
-
+      /* restore child_handler */
+      signal (SIGCHLD, &child_handler);
     }
     if (c->container)
     {
@@ -499,6 +508,11 @@ void destroy_call (struct call *c)
 
     free (c);
 
+    /*
+     * Signal child_handler just to be sure that waitpid is done if some
+     * pppd's died meanwhile
+     */
+    kill (SIGCHLD, getpid ());
 }
 
 
